@@ -25,7 +25,7 @@ namespace aKode {
 
 EncoderPluginHandler::EncoderPluginHandler(const string lib) : encoder_plugin(0)
 {
-    if (lib.size() > 0);
+    if (lib.size() > 0)
         load(lib);
 }
 
@@ -39,7 +39,8 @@ bool EncoderPluginHandler::load(const string name)
     return res && encoder_plugin;
 }
 
-Encoder* EncoderPluginHandler::openEncoder(File *dest) {
+Encoder* EncoderPluginHandler::openEncoder(File *dest)
+{
     if (encoder_plugin)
         return encoder_plugin->openEncoder(dest);
     else
