@@ -19,7 +19,10 @@
 #include <qslider.h>
 #include <qtimer.h>
 
+#include <math.h>
+
 #include "artsplayer.h"
+#include "akodeplayer.h"
 #include "gstreamerplayer.h"
 #include "playermanager.h"
 #include "playlistinterface.h"
@@ -35,30 +38,42 @@ using namespace ActionCollection;
 
 PlayerManager *PlayerManager::m_instance = 0;
 
+enum PlayerManagerStatus { StatusStopped = -1, StatusPaused = 1, StatusPlaying = 2 };
+
 ////////////////////////////////////////////////////////////////////////////////
 // helper functions
 ////////////////////////////////////////////////////////////////////////////////
 
-enum SoundSystem { ArtsBackend = 0, GStreamerBackend = 1 };
+enum SoundSystem { ArtsBackend = 0, GStreamerBackend = 1, AkodeBackend = 2 };
 
 static Player *createPlayer(int system = ArtsBackend)
 {
 
     Player *p = 0;
-#if HAVE_GSTREAMER
     switch(system) {
+    case AkodeBackend:
+        p = new aKodePlayer;
+        break;
+#if HAVE_ARTS
     case ArtsBackend:
         p = new ArtsPlayer;
         break;
+#endif
+#if HAVE_GSTREAMER
     case GStreamerBackend:
         p = new GStreamerPlayer;
         break;
-    }
-#else
-    Q_UNUSED(system);
-    p = new ArtsPlayer;
 #endif
-
+    default:
+#if HAVE_ARTS
+        p = new ArtsPlayer;
+#elif HAVE_GSTREAMER
+        p = new GStreamerPlayer;
+#else
+        p = new aKodePlayer;
+#endif
+        break;
+    }
     return p;
 }
 
@@ -125,19 +140,15 @@ float PlayerManager::volume() const
 
 int PlayerManager::status() const
 {
-    // DCOP call to interface better with the Media Control applet
-    // Constant values provided by mETz.
-    // mETz: You can thank me later. :-)
-
     if(!player())
-        return -1;
+        return StatusStopped;
 
     if(player()->paused())
-        return 1;
+        return StatusPaused;
 
     if(player()->playing())
-        return 2;
-    
+        return StatusPlaying;
+
     return 0;
 }
 
@@ -204,14 +215,17 @@ void PlayerManager::setStatusLabel(StatusLabel *label)
 KSelectAction *PlayerManager::playerSelectAction(QObject *parent) // static
 {
     KSelectAction *action = 0;
-#if HAVE_GSTREAMER
     action = new KSelectAction(i18n("&Output To"), 0, parent, "outputSelect");
     QStringList l;
-    l << "aRts" << "GStreamer";
-    action->setItems(l);
-#else
-    Q_UNUSED(parent);
+    l <<
+#if HAVE_ARTS
+        i18n("aRts") <<
 #endif
+#if HAVE_GSTREAMER
+        i18n("GStreamer") <<
+#endif
+        i18n("aKode");
+    action->setItems(l);
     return action;
 }
 
@@ -228,11 +242,12 @@ void PlayerManager::play(const FileHandle &file)
         if(player()->paused())
             player()->play();
         else if(player()->playing()) {
-            m_sliderAction->trackPositionSlider()->setValue(0);
+            if(m_sliderAction->trackPositionSlider())
+                m_sliderAction->trackPositionSlider()->setValue(0);
             player()->seekPosition(0);
         }
         else {
-	    m_playlistInterface->playNext();
+            m_playlistInterface->playNext();
             m_file = m_playlistInterface->currentFile();
 
             if(!m_file.isNull())
@@ -254,9 +269,12 @@ void PlayerManager::play(const FileHandle &file)
     action("pause")->setEnabled(true);
     action("stop")->setEnabled(true);
     action("forward")->setEnabled(true);
+    if(action<KToggleAction>("albumRandomPlay")->isChecked())
+        action("forwardAlbum")->setEnabled(true);
     action("back")->setEnabled(true);
 
-    m_sliderAction->trackPositionSlider()->setEnabled(true);
+    if(m_sliderAction->trackPositionSlider())
+        m_sliderAction->trackPositionSlider()->setEnabled(true);
 
     m_timer->start(m_pollInterval);
 
@@ -306,9 +324,12 @@ void PlayerManager::stop()
     action("stop")->setEnabled(false);
     action("back")->setEnabled(false);
     action("forward")->setEnabled(false);
+    action("forwardAlbum")->setEnabled(false);
 
-    m_sliderAction->trackPositionSlider()->setValue(0);
-    m_sliderAction->trackPositionSlider()->setEnabled(false);
+    if(m_sliderAction->trackPositionSlider()) {
+        m_sliderAction->trackPositionSlider()->setValue(0);
+        m_sliderAction->trackPositionSlider()->setEnabled(false);
+    }
 
     player()->stop();
     m_playlistInterface->stop();
@@ -344,19 +365,19 @@ void PlayerManager::seekPosition(int position)
 
     slotUpdateTime(position);
     player()->seekPosition(position);
-    m_sliderAction->trackPositionSlider()->setValue(position);
+
+    if(m_sliderAction->trackPositionSlider())
+        m_sliderAction->trackPositionSlider()->setValue(position);
 }
 
 void PlayerManager::seekForward()
 {
-    int position = m_sliderAction->trackPositionSlider()->value();
-    seekPosition(kMin(m_sliderAction->trackPositionSlider()->maxValue(), position + 10));
+    seekPosition(kMin(SliderAction::maxPosition, position() + 10));
 }
 
 void PlayerManager::seekBack()
 {
-    int position = m_sliderAction->trackPositionSlider()->value();
-    seekPosition(kMax(m_sliderAction->trackPositionSlider()->minValue(), position - 10));
+    seekPosition(kMax(SliderAction::minPosition, position() - 10));
 }
 
 void PlayerManager::playPause()
@@ -388,7 +409,7 @@ void PlayerManager::back()
 
 void PlayerManager::volumeUp()
 {
-    if(!player() || !m_sliderAction)
+    if(!player() || !m_sliderAction || !m_sliderAction->volumeSlider())
         return;
 
     int volume = m_sliderAction->volumeSlider()->volume() +
@@ -400,7 +421,7 @@ void PlayerManager::volumeUp()
 
 void PlayerManager::volumeDown()
 {
-    if(!player() || !m_sliderAction)
+    if(!player() || !m_sliderAction || !m_sliderAction->volumeSlider())
         return;
 
     int volume = m_sliderAction->volumeSlider()->value() -
@@ -412,7 +433,7 @@ void PlayerManager::volumeDown()
 
 void PlayerManager::mute()
 {
-    if(!player() || !m_sliderAction)
+    if(!player() || !m_sliderAction || !m_sliderAction->volumeSlider())
         return;
 
     slotSetVolume(m_muted ? m_sliderAction->volumeSlider()->value() : 0);
@@ -441,7 +462,8 @@ void PlayerManager::slotPollPlay()
             stop();
     }
     else if(!m_sliderAction->dragging()) {
-        m_sliderAction->trackPositionSlider()->setValue(player()->position());
+        if(m_sliderAction->trackPositionSlider())
+            m_sliderAction->trackPositionSlider()->setValue(player()->position());
 
         if(m_statusLabel) {
             m_statusLabel->setItemTotalTime(player()->totalTime());
@@ -449,6 +471,23 @@ void PlayerManager::slotPollPlay()
         }
     }
 
+    // This call is done because when the user adds the slider to the toolbar
+    // while playback is occuring the volume slider generally defaults to 0,
+    // and doesn't get updated to the correct volume.  It might be better to
+    // have the SliderAction class fill in the correct volume, but I'm trying
+    // to avoid having it depend on PlayerManager since it may not be
+    // constructed in time during startup. -mpyne
+
+    if(!m_sliderAction->volumeDragging() && m_sliderAction->volumeSlider())
+    {
+        int maxV = m_sliderAction->volumeSlider()->maxValue();
+        float v = sqrt(sqrt(volume())); // Cancel out exponential scaling
+
+        m_sliderAction->volumeSlider()->blockSignals(true);
+        m_sliderAction->volumeSlider()->setVolume((int)((v) * maxV));
+        m_sliderAction->volumeSlider()->blockSignals(false);
+    }
+
     // Ok, this is weird stuff, but it works pretty well.  Ordinarily we don't
     // need to check up on our playing time very often, but in the span of the
     // last interval, we want to check a lot -- to figure out that we've hit the
@@ -464,17 +503,41 @@ void PlayerManager::slotPollPlay()
     m_noSeek = false;
 }
 
-void PlayerManager::slotSetOutput(int system)
+void PlayerManager::slotSetOutput(const QString &system)
 {
     stop();
-    delete m_player;
-    m_player = createPlayer(system);
+    setOutput(system);
     setup();
 }
 
+void PlayerManager::setOutput(const QString &system)
+{
+    delete m_player;
+    if(system == i18n("aRts"))
+        m_player = createPlayer(ArtsBackend);
+    else if(system == i18n("GStreamer"))
+        m_player = createPlayer(GStreamerBackend);
+    else if(system == i18n("aKode"))
+        m_player = createPlayer(AkodeBackend);
+}
+
 void PlayerManager::slotSetVolume(int volume)
 {
-    setVolume(float(volume) / float(m_sliderAction->volumeSlider()->maxValue()));
+    float scaledVolume;
+
+    if(m_sliderAction->volumeSlider())
+        scaledVolume = float(volume) / m_sliderAction->volumeSlider()->maxValue();
+    else {
+        scaledVolume = float(volume) / 100.0; // Hopefully this is accurate
+        scaledVolume = kMin(1.0f, scaledVolume);
+    }
+
+    // Perform exponential scaling to counteract the fact that humans perceive
+    // volume changes logarithmically.
+
+    scaledVolume *= scaledVolume;
+    scaledVolume *= scaledVolume;
+    setVolume(scaledVolume); // scaledVolume ^ 4
 }
 
 void PlayerManager::slotUpdateTime(int position)
@@ -482,7 +545,7 @@ void PlayerManager::slotUpdateTime(int position)
     if(!m_statusLabel)
         return;
 
-    float positionFraction = float(position) / m_sliderAction->trackPositionSlider()->maxValue();
+    float positionFraction = float(position) / SliderAction::maxPosition;
     float totalTime = float(player()->totalTime());
     int seekTime = int(positionFraction * totalTime + 0.5); // "+0.5" for rounding
 
@@ -508,6 +571,7 @@ void PlayerManager::setup()
     if(!action("pause") ||
        !action("stop") ||
        !action("back") ||
+       !action("forwardAlbum") ||
        !action("forward") ||
        !action("trackPositionAction"))
     {
@@ -521,6 +585,7 @@ void PlayerManager::setup()
     action("stop")->setEnabled(false);
     action("back")->setEnabled(false);
     action("forward")->setEnabled(false);
+    action("forwardAlbum")->setEnabled(false);
 
     // setup sliders
 
@@ -530,24 +595,29 @@ void PlayerManager::setup()
             this, SLOT(seekPosition(int)));
     connect(m_sliderAction->trackPositionSlider(), SIGNAL(valueChanged(int)),
             this, SLOT(slotUpdateTime(int)));
-    connect(m_sliderAction->volumeSlider(), SIGNAL(signalVolumeChanged(int)),
+    connect(m_sliderAction, SIGNAL(signalVolumeChanged(int)),
             this, SLOT(slotSetVolume(int)));
 
     // Call this method manually to avoid warnings.
-    
+
     KAction *outputAction = actions()->action("outputSelect");
 
     if(outputAction) {
-        int mediaSystem = static_cast<KSelectAction *>(outputAction)->currentItem();
-        m_player = createPlayer(mediaSystem);
-        connect(outputAction, SIGNAL(activated(int)), this, SLOT(slotSetOutput(int)));
+        setOutput(static_cast<KSelectAction *>(outputAction)->currentText());
+        connect(outputAction, SIGNAL(activated(const QString &)), this, SLOT(slotSetOutput(const QString &)));
     }
     else
         m_player = createPlayer();
 
-    float volume =
-        float(m_sliderAction->volumeSlider()->volume()) /
-        float(m_sliderAction->volumeSlider()->maxValue());
+    float volume;
+    
+    if(m_sliderAction->volumeSlider()) {
+        volume = 
+            float(m_sliderAction->volumeSlider()->volume()) /
+            float(m_sliderAction->volumeSlider()->maxValue());
+    }
+    else
+        volume = 1; // Assume user wants full volume
 
     m_player->setVolume(volume);
 
@@ -556,3 +626,5 @@ void PlayerManager::setup()
 }
 
 #include "playermanager.moc"
+
+// vim: set et ts=4 sw=4:
