@@ -128,6 +128,11 @@ void SearchLine::clear()
 	m_lineEdit->clear();
 }
 
+void SearchLine::setFocus()
+{
+    m_lineEdit->setFocus();
+}
+
 void SearchLine::slotActivate()
 {
     action("stop")->activate();
@@ -193,6 +198,16 @@ void SearchWidget::setSearch(const PlaylistSearch &search)
     m_searchLine->setSearchComponent(*components.begin());
 }
 
+QString SearchWidget::searchText() const
+{
+    return m_searchLine->searchComponent().query();
+}
+
+void SearchWidget::setSearchText(const QString &text)
+{
+    m_searchLine->setSearchComponent(PlaylistSearch::Component(text));
+}
+
 PlaylistSearch SearchWidget::search(const PlaylistList &playlists) const
 {
     PlaylistSearch::ComponentList components;
@@ -200,6 +215,8 @@ PlaylistSearch SearchWidget::search(const PlaylistList &playlists) const
     return PlaylistSearch(playlists, components);
 }
 
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // SearchWidget public slots
 ////////////////////////////////////////////////////////////////////////////////
@@ -215,6 +232,11 @@ void SearchWidget::setEnabled(bool enable)
     setShown(enable);
 }
 
+void SearchWidget::setFocus()
+{
+    m_searchLine->setFocus();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // SearchWidget private methods
 ////////////////////////////////////////////////////////////////////////////////
@@ -239,11 +261,15 @@ void SearchWidget::setupLayout()
     connect(clearSearchButton, SIGNAL(pressed()), m_searchLine, SLOT(clear()));
     setStretchableWidget(m_searchLine);
 
-    QToolButton *b = new QToolButton(this);
-    b->setTextLabel(i18n("Advanced Search"), true);
-    b->setIconSet(SmallIconSet("wizard"));
+    // I've decided that I think this is ugly, for now.
 
-    connect(b, SIGNAL(clicked()), this, SIGNAL(signalAdvancedSearchClicked()));
+    /*
+      QToolButton *b = new QToolButton(this);
+      b->setTextLabel(i18n("Advanced Search"), true);
+      b->setIconSet(SmallIconSet("wizard"));
+
+      connect(b, SIGNAL(clicked()), this, SIGNAL(signalAdvancedSearchClicked()));
+    */
 }
 
 #include "searchwidget.moc"
