@@ -1,12 +1,11 @@
 /*
  * KMix -- KDE's full featured mini mixer
  *
- * $Id: kmix.cpp,v 1.171.2.1 2004/02/10 00:26:51 englich Exp $
+ * $Id: kmix.cpp 408616 2005-04-29 13:41:05Z lunakl $
  *
  * Copyright (C) 2000 Stefan Schimanski <schimmi@kde.org>
  * Copyright (C) 2001 Preston Brown <pbrown@kde.org>
  * Copyright (C) 2003 Sven Leiber <s.leiber@web.de>
- *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -25,8 +24,8 @@
 // include files for QT
 #include <qmap.h>
 #include <qhbox.h>
-#include <qtimer.h>
 #include <qcheckbox.h>
+#include <qradiobutton.h>
 #include <qwidgetstack.h>
 #include <qlayout.h>
 #include <qtooltip.h>
@@ -47,32 +46,34 @@
 #include <khelpmenu.h>
 #include <kdebug.h>
 #include <kaccel.h>
-#include <kkeydialog.h>
 #include <kpopupmenu.h>
 
-	// application specific includes
+// application specific includes
+#include "mixertoolbox.h"
 #include "kmix.h"
 #include "kmixerwidget.h"
 #include "kmixprefdlg.h"
 #include "kmixdockwidget.h"
+#include "kmixtoolbox.h"
 
 
 /**
  * Constructs a mixer window (KMix main window)
  */
 KMixWindow::KMixWindow()
-	: KMainWindow(0, 0, 0 ), m_showTicks( false ), m_maxId( 0 ),
+	: KMainWindow(0, 0, 0, 0), m_showTicks( true ),
 	m_lockedLayout(0),
 	m_dockWidget( 0L )
 {
 	m_visibilityUpdateAllowed	= true;
 	m_multiDriverMode		= false; // -<- I never-ever want the multi-drivermode to be activated by accident
+	m_surroundView		        = false; // -<- The same is true for the experimental surround View (3D)
 	// As long as we do not know better, we assume to start hidden. We need
 	// to initialize this variable here, as we don't trigger a hideEvent().
 	m_isVisible = false;
 	m_mixerWidgets.setAutoDelete(true);
 	loadConfig(); // Need to load config before initMixer(), due to "MultiDriver" keyword
-	initMixer();
+	MixerToolBox::initMixer(m_mixers, m_multiDriverMode, m_hwInfoString);
 	initActions();
 	initWidgets();
 	initMixerWidgets();
@@ -93,8 +94,6 @@ KMixWindow::KMixWindow()
 		hide();
 	}
 	connect( kapp, SIGNAL( aboutToQuit()), SLOT( saveSettings()) );
-
-	//setMaximumSize( QSize( 32767, size().height() ) );  // !!! What is this good for ?!?
 }
 
 
@@ -112,135 +111,17 @@ KMixWindow::initActions()
 	// settings menu
 	KStdAction::showMenubar( this, SLOT(toggleMenuBar()), actionCollection());
 	KStdAction::preferences( this, SLOT(showSettings()), actionCollection());
+	KStdAction::keyBindings( guiFactory(), SLOT(configureShortcuts()), actionCollection());
 
-	KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
-
+/* Belongs in KMixDockWidget
 	(void)new KToggleAction( i18n( "M&ute" ), 0, this, SLOT( dockMute() ),
 				 actionCollection(), "dock_mute" );
-
+*/
 	(void) new KAction( i18n( "Hardware &Information" ), 0, this, SLOT( slotHWInfo() ), actionCollection(), "hwinfo" );
+	(void) new KAction( i18n( "Hide Mixer Window" ), Key_Escape, this, SLOT(hide()), actionCollection(), "hide_kmixwindow" );
 	createGUI( "kmixui.rc" );
 }
 
-void KMixWindow::slotConfigureKeys()
-{
-  KKeyDialog::configure( actionCollection(), this );
-}
-
-void
-KMixWindow::initMixer()
-{
-	QString tmpstr;
-	timer = new QTimer( this );  // timer will be started on show()
-
-	// get maximum values
-	KConfig *config= new KConfig("kcmkmixrc", false);
-	config->setGroup("Misc");
-	int maxDevices = config->readNumEntry( "maxDevices", 2 );
-	delete config;
-
-	// poll for mixers
-	QMap<QString,int> mixerNums;
-	int drvNum = Mixer::getDriverNum();
-
-	int driverWithMixer = -1;
-	bool multipleDriversActive = false;
-
-	QString driverInfo = "";
-	QString driverInfoUsed = "";
-	for( int drv=0; drv<drvNum ; drv++ )
-	{
-		QString driverName = Mixer::driverName(drv);
-		if ( drv!= 0 )
-		{
-			driverInfo += " + ";
-		}
-		driverInfo += driverName;
-	}
-
-	/* Run a loop over all drivers. The loop will terminate after the first driver which
-	   has mixers. And here is the reason:
-	   - If you run ALSA with ALSA-OSS-Emulation enabled, mixers will show up twice: once
-	     as native ALSA mixer, once as OSS mixer (emulated by ALSA). This is bad and WILL
-	     confuse users. So it is a design decision that we can compile in multiple drivers
-	     but we can run only one driver.
-	   - For special usage scenarios, people will still want to run both drivers at the
-	     same time. We allow them to hack their Config-File, where they can enable a
-	     multi-driver mode.
-	   - Another remark: For KMix3.0 or so, we should allow multiple-driver, for allowing
-	     addition of special-use drivers, e.g. an ARTS-mixer-driver, or a CD-Rom volume driver.
-	 */
-	for( int drv=0; drv<drvNum && ( m_multiDriverMode || m_mixers.count()==0) ; drv++ )
-	{
-		bool drvInfoAppended = false;
-		{
-			for( int dev=0; dev<maxDevices; dev++ )
-			{
-				// hint: no driver is using cardnum, except the unsupported alsa0.5 - esken
-				for( int card=0; card<1 /*maxCards*/; card++ )
-				{
-					Mixer *mixer = Mixer::getMixer( drv, dev, card );
-					int mixerError = mixer->grab();
-					if ( mixerError!=0 )
-					{
-						delete mixer;
-						continue;
-					}
-
-					connect( timer, SIGNAL(timeout()), mixer, SLOT(readSetFromHW()));
-					m_mixers.append( mixer );
-					mixer->readSetFromHW();
-
-					// append driverName (used drivers)
-					if ( !drvInfoAppended )
-					{
-						drvInfoAppended = true;
-						QString driverName = Mixer::driverName(drv);
-						if ( drv!= 0 )
-						{
-							driverInfoUsed += " + ";
-						}
-						driverInfoUsed += driverName;
-					}
-
-					// Check whether there are mixers in different drivers, so that the usr can be warned
-					if (!multipleDriversActive)
-					{
-						if ( driverWithMixer == -1 )
-						{
-							// Aha, this is the very first detected device
-							driverWithMixer = drv;
-						}
-						else
-						{
-							if ( driverWithMixer != drv )
-							{
-								// Got him: There are mixers in different drivers
-								multipleDriversActive = true;
-							}
-						}
-					}
-
-					// count mixer nums for every mixer name to identify mixers with equal names
-					mixerNums[mixer->mixerName()]++;
-					mixer->setMixerNum( mixerNums[mixer->mixerName()] );
-				}
-			}
-		}
-	}
-
-	m_hwInfoString = i18n("Sound drivers supported");
-	m_hwInfoString += ": " + driverInfo +
-		"\n" + i18n("Sound drivers used") + ": " + driverInfoUsed;
-	if ( multipleDriversActive )
-	{
-		// this will only be possible by hacking the config-file, as it will not be officially supported
-		m_hwInfoString += "\nExperimental multiple-Driver mode activated";
-	}
-
-	kdDebug() << m_hwInfoString << endl;
-}
-
 
 void
 KMixWindow::initPrefDlg()
@@ -259,13 +140,19 @@ KMixWindow::initWidgets()
 
 	// Widgets layout
 	widgetsLayout = new QVBoxLayout(   centralWidget(), 0, 0, "widgetsLayout" );
-	widgetsLayout->setResizeMode(QLayout::Minimum); // basically good, but needs more work
+	widgetsLayout->setResizeMode(QLayout::Minimum); // works fine
+
 
 	// Mixer widget line
 	mixerNameLayout = new QHBox( centralWidget(), "mixerNameLayout" );
+        widgetsLayout->setStretchFactor( mixerNameLayout, 0 );
+	QSizePolicy qsp( QSizePolicy::Ignored, QSizePolicy::Maximum);
+	mixerNameLayout->setSizePolicy(qsp);
 	mixerNameLayout->setSpacing(KDialog::spacingHint());
-	new QLabel( i18n(" Current mixer:"), mixerNameLayout );
+	QLabel *qlbl = new QLabel( i18n(" Current mixer:"), mixerNameLayout );
+	qlbl->setFixedHeight(qlbl->sizeHint().height());
 	m_cMixer = new KComboBox( FALSE, mixerNameLayout, "mixerCombo" );
+	m_cMixer->setFixedHeight(m_cMixer->sizeHint().height());
 	connect( m_cMixer, SIGNAL( activated( int ) ), this, SLOT( showSelectedMixer( int ) ) );
 	QToolTip::add( m_cMixer, i18n("Current mixer" ) );
 
@@ -273,6 +160,7 @@ KMixWindow::initWidgets()
 	widgetsLayout->addWidget( mixerNameLayout );
 
 	m_wsMixers = new QWidgetStack( centralWidget(), "MixerWidgetStack" );
+        widgetsLayout->setStretchFactor( m_wsMixers, 10 );
 	widgetsLayout->addWidget( m_wsMixers );
 
 	if ( m_showMenubar )
@@ -280,6 +168,7 @@ KMixWindow::initWidgets()
 	else
 		menuBar()->hide();
 
+	widgetsLayout->activate();
 }
 
 
@@ -297,70 +186,29 @@ KMixWindow::updateDocking()
 	{
 
 		// create dock widget
-		m_dockWidget = new KMixDockWidget( m_mixers.first(), this, "mainDockWidget" );
-		updateDockIcon();
+                // !! This should be a View in the future
+		m_dockWidget = new KMixDockWidget( m_mixers.first(), this, "mainDockWidget", m_volumeWidget );
 
+/* Belongs in KMixDockWidget
 		// create RMB menu
 		KPopupMenu *menu = m_dockWidget->contextMenu();
 
-		KAction *a = actionCollection()->action("options_configure");
-		if (a) a->plug( menu );
-
-		a = actionCollection()->action("help_about_app");
-		if (a) a->plug( menu );
-
-		a = actionCollection()->action("help");
-		if (a) a->plug( menu );
-
-		menu->insertSeparator();
-
-		a = actionCollection()->action( "dock_mute" );
+		// !! check this
+		KAction *a = actionCollection()->action( "dock_mute" );
 		if ( a ) a->plug( menu );
+*/
+
+		/*
+		 * Mail from 31.1.2005: "make sure your features are at least string complete"
+		 * Preparation for fixing Bug #55078 - scheduled for KDE3.4.1 .
+		 * This text will be plugged into the dock-icon popup menu.
+		 */
+		QString selectChannel = i18n("Select Channel"); // This text will be used in KDE3.4.1 !!!
 
 		m_dockWidget->show();
 	}
 }
 
-void
-KMixWindow::dockMute()
-{
-	Mixer *mixer = m_mixers.first();
-        if ( !mixer )
-            return;
-	MixDevice *masterDevice = ( *mixer )[ mixer->masterDevice() ];
-
-	masterDevice->setMuted( !masterDevice->isMuted() );
-	mixer->writeVolumeToHW( masterDevice->num(), masterDevice->getVolume() );
-
-	updateDockIcon();
-}
-
-void
-KMixWindow::updateDockIcon()
-{
-	Mixer *mixer = m_mixers.first();
-        if ( !mixer )
-	{
-	    m_dockWidget->setErrorPixmap();
-            return;
-	}
-	MixDevice *masterDevice = ( *mixer )[ mixer->masterDevice() ];
-
-	// Required if muted from mixer widget
-	KToggleAction *a = static_cast<KToggleAction *>( actionCollection()->action(  "dock_mute" ) );
-	if ( a ) a->setChecked( masterDevice->isMuted() );
-
-	m_dockWidget->updatePixmap();
-	m_dockWidget->setVolumeTip( 0, masterDevice->getVolume() );
-}
-
-void
-KMixWindow::updateDockTip(Volume vol)
-{
-   if ( m_dockWidget )
-	m_dockWidget->setVolumeTip( 0, vol );
-}
-
 void
 KMixWindow::saveSettings()
 {
@@ -371,8 +219,8 @@ KMixWindow::saveSettings()
 void
 KMixWindow::saveConfig()
 {
-	KConfig *config = kapp->config();
-	config->setGroup(0);
+    KConfig *config = kapp->config();
+    config->setGroup(0);
 
    // make sure we don't start without any UI
    // can happen e.g. when not docked and kmix closed via 'X' button
@@ -380,7 +228,7 @@ KMixWindow::saveConfig()
    if ( !m_showDockWidget )
        startVisible = true;
 
-	config->writeEntry( "Size", size() );
+   config->writeEntry( "Size", size() );
    config->writeEntry( "Position", pos() );
    // Cannot use isVisible() here, as in the "aboutToQuit()" case this widget is already hidden.
    // (Please note that the problem was only there when quitting via Systray - esken).
@@ -390,41 +238,44 @@ KMixWindow::saveConfig()
    config->writeEntry( "TrayVolumeControl", m_volumeWidget );
    config->writeEntry( "Tickmarks", m_showTicks );
    config->writeEntry( "Labels", m_showLabels );
+   config->writeEntry( "startkdeRestore", m_onLogin );
+   if ( m_toplevelOrientation  == Qt::Vertical )
+      config->writeEntry( "Orientation","Vertical" );
+   else
+      config->writeEntry( "Orientation","Horizontal" );
 
    // save mixer widgets
-   QStringList devices;
    for ( KMixerWidget *mw = m_mixerWidgets.first(); mw != 0; mw = m_mixerWidgets.next() )
    {
 		QString grp;
 		grp.sprintf( "%i", mw->id() );
-		devices << grp;
-
-		config->setGroup( grp );
-		config->writeEntry( "Mixer", mw->mixerNum() );
-		config->writeEntry( "MixerName", mw->mixerName() );
-		config->writeEntry( "Name", mw->name() );
-
 		mw->saveConfig( config, grp );
-	}
+   }
 
    config->setGroup(0);
-   config->writeEntry( "Devices", devices );
 }
 
 void
 KMixWindow::loadConfig()
 {
-	KConfig *config = kapp->config();
-	config->setGroup(0);
+    KConfig *config = kapp->config();
+    config->setGroup(0);
 
    m_showDockWidget = config->readBoolEntry("AllowDocking", true);
    m_volumeWidget = config->readBoolEntry("TrayVolumeControl", true);
 	//hide on close has to stay true for usability. KMixPrefDlg option commented out. nolden
    m_hideOnClose = config->readBoolEntry("HideOnClose", true);
-   m_showTicks = config->readBoolEntry("Tickmarks", false);
-   m_showLabels = config->readBoolEntry("Labels", false);
+   m_showTicks = config->readBoolEntry("Tickmarks", true);
+   m_showLabels = config->readBoolEntry("Labels", true);
+   m_onLogin = config->readBoolEntry("startkdeRestore", true );
    m_startVisible = config->readBoolEntry("Visible", true);
    m_multiDriverMode = config->readBoolEntry("MultiDriver", false);
+   m_surroundView    = config->readBoolEntry("Experimental-ViewSurround", false );
+   const QString& orientationString = config->readEntry("Orientation", "Horizontal");
+   if ( orientationString == "Vertical" )
+       m_toplevelOrientation  = Qt::Vertical;
+   else
+       m_toplevelOrientation = Qt::Horizontal;
 
    // show/hide menu bar
    m_showMenubar = config->readBoolEntry("Menubar", true);
@@ -454,23 +305,40 @@ KMixWindow::initMixerWidgets()
 	int id=0;
 	Mixer *mixer;
 
+	// Attention!! If m_mixers is empty, we behave stupid. We don't show nothing and there
+        //             is not even a context menu.
 	for ( mixer=m_mixers.first(),id=0; mixer!=0; mixer=m_mixers.next(),id++ )
 	{
-		kdDebug() << "Mixer number: " << id << " Name: " << mixer->mixerName() << endl ;
+	    //kdDebug(67100) << "Mixer number: " << id << " Name: " << mixer->mixerName() << endl ;
 
 
+                ViewBase::ViewFlags vflags = ViewBase::HasMenuBar;
+                if ( m_showMenubar ) {
+                    vflags |= ViewBase::MenuBarVisible;
+	        }
+		if (  m_surroundView ) {
+		    vflags |= ViewBase::Experimental_SurroundView;
+		}
+		if ( m_toplevelOrientation == Qt::Vertical ) {
+		    vflags |= ViewBase::Vertical;
+		}
+		else {
+		    vflags |= ViewBase::Horizontal;
+		}
+
+	
 		KMixerWidget *mw = new KMixerWidget( id, mixer, mixer->mixerName(), mixer->mixerNum(),
-				false, KPanelApplet::Up, MixDevice::ALL, this, "KMixerWidget" );
+						     MixDevice::ALL, this, "KMixerWidget", vflags );
 
-		mw->setName( mixer->mixerName() );
+		//mw->setName( mixer->mixerName() );
 
 		m_mixerWidgets.append( mw );
 
 		// Add to Combo
-		m_cMixer->insertItem( mw->name() );
+		m_cMixer->insertItem( mixer->mixerName() );
 
 		// Add to Stack
-		kdDebug() << "Inserted mixer " << id << ":" << mw->name() << endl;
+		//kdDebug(67100) << "Inserted mixer " << id << ":" << mw->name() << endl;
 		m_wsMixers->addWidget( mw, id );
 
 		QString grp;
@@ -479,16 +347,8 @@ KMixWindow::initMixerWidgets()
 
 		mw->setTicks( m_showTicks );
 		mw->setLabels( m_showLabels );
-		mw->addActionToPopup( actionCollection()->action("options_show_menubar") );
+		// !! I am still not sure whether this works 100% reliably - chris
 		mw->show();
-
-		// ulgy hack to avoid sending to many updateSize requests to kicker that would freeze it
-		m_layoutTimer = new QTimer( this );
-		connect( m_layoutTimer, SIGNAL(timeout()), this, SLOT(updateLayoutNow()) );
-
-		connect( mw, SIGNAL( masterMuted( bool ) ), SLOT( updateDockIcon() ) );
-		connect( mw, SIGNAL( newMasterVolume( Volume ) ), SLOT( updateDockTip(Volume) ) );
-		connect( mw, SIGNAL(updateLayout()), this, SLOT(triggerUpdateLayout()));
 	}
 
 	if (id == 1)
@@ -503,10 +363,10 @@ KMixWindow::queryClose ( )
 {
     if ( m_showDockWidget && !kapp->sessionSaving() )
     {
-		  hide();
-		 return false;
-	 }
-	 return true;
+        hide();
+        return false;
+    }
+    return true;
 }
 
 
@@ -516,35 +376,19 @@ KMixWindow::quit()
 	kapp->quit();
 }
 
-// method "stolen" from KMixApplet
-void KMixWindow::triggerUpdateLayout()
-{
-	if ( m_lockedLayout ) return;
-	if ( !m_layoutTimer->isActive() ) {
-		m_layoutTimer->start( 100, TRUE );
-	}
-}
-
-// method "stolen" from KMixApplet
-void KMixWindow::updateLayoutNow()
-{
-   m_lockedLayout++;
- 	resize(widgetsLayout->minimumSize() );
-   //emit updateLayout();
-   //saveConfig(); // ugly hack to get the config saved somehow
-   m_lockedLayout--;
-}
-
 
 void
 KMixWindow::showSettings()
 {
-	if (!m_prefDlg->isVisible())
+   if (!m_prefDlg->isVisible())
    {
       m_prefDlg->m_dockingChk->setChecked( m_showDockWidget );
       m_prefDlg->m_volumeChk->setChecked(m_volumeWidget);
       m_prefDlg->m_showTicks->setChecked( m_showTicks );
       m_prefDlg->m_showLabels->setChecked( m_showLabels );
+      m_prefDlg->m_onLogin->setChecked( m_onLogin );
+      m_prefDlg->_rbVertical  ->setChecked( m_toplevelOrientation == Qt::Vertical );
+      m_prefDlg->_rbHorizontal->setChecked( m_toplevelOrientation == Qt::Horizontal );
 
       m_prefDlg->show();
    }
@@ -565,10 +409,12 @@ KMixWindow::showAbout()
 }
 
 /**
- * Loads the volumes of all mixers from kmixctrl. In other words:
+ * Loads the volumes of all mixers from kmixctrlrc.
+ * In other words:
  * Restores the default voumes as stored via saveVolumes() or the
- * Mixer control center module..
+ * execution of "kmixctrl --save"
  */
+/* Currently this is not in use
 void
 KMixWindow::loadVolumes()
 {
@@ -579,7 +425,7 @@ KMixWindow::loadVolumes()
 	}
 	delete cfg;
 }
-
+*/
 
 /**
  * Stores the volumes of all mixers  Can be restored via loadVolumes() or
@@ -588,12 +434,12 @@ KMixWindow::loadVolumes()
 void
 KMixWindow::saveVolumes()
 {
-	KConfig *cfg = new KConfig( "kmixctrlrc", false );
-   for (Mixer *mixer=m_mixers.first(); mixer!=0; mixer=m_mixers.next())
-	{
-      mixer->volumeSave( cfg );
-	}
-   delete cfg;
+    KConfig *cfg = new KConfig( "kmixctrlrc", false );
+    for (Mixer *mixer=m_mixers.first(); mixer!=0; mixer=m_mixers.next()) {
+	//kdDebug(67100) << "KMixWindow::saveConfig()" << endl;
+	mixer->volumeSave( cfg );
+    }
+    delete cfg;
 }
 
 
@@ -604,23 +450,34 @@ KMixWindow::applyPrefs( KMixPrefDlg *prefDlg )
    m_volumeWidget = prefDlg->m_volumeChk->isChecked();
    m_showTicks = prefDlg->m_showTicks->isChecked();
    m_showLabels = prefDlg->m_showLabels->isChecked();
+   m_onLogin = prefDlg->m_onLogin->isChecked();
+   if ( prefDlg->_rbVertical->isChecked() ) {
+      kdDebug(67100) << "KMix should change to Vertical layout\n";
+      m_toplevelOrientation = Qt::Vertical;
+   }
+   else if ( prefDlg->_rbHorizontal->isChecked() ) {
+     kdDebug(67100) << "KMix should change to Horizontal layout\n";
+     m_toplevelOrientation = Qt::Horizontal;
+   }
+
 
    this->setUpdatesEnabled(false);
+   updateDocking();
+
    for (KMixerWidget *mw=m_mixerWidgets.first(); mw!=0; mw=m_mixerWidgets.next())
    {
       mw->setTicks( m_showTicks );
       mw->setLabels( m_showLabels );
+      mw->mixer()->readSetFromHWforceUpdate(); // needed, as updateDocking() has reconstructed the DockWidget
    }
 
-   updateDocking();
    this->setUpdatesEnabled(false);
 
    // avoid invisible and unaccessible main window
    if( !m_showDockWidget && !isVisible() )
-	{
-      timer->start(500);  // !!! Lets see how context menu, tooltips work
-      show();
-	}
+   {
+       show();
+   }
 
    this->repaint(); // make KMix look fast (saveConfig() often uses several seconds)
    kapp->processEvents();
@@ -646,18 +503,20 @@ void
 KMixWindow::showEvent( QShowEvent * )
 {
     if ( m_visibilityUpdateAllowed )
-		 m_isVisible = true;
-	 timer->start(500);
+	m_isVisible = isVisible();
+    // !! could possibly start polling now (idea: use someting like ref() and unref() on Mixer instance
 }
 
 void
 KMixWindow::hideEvent( QHideEvent * )
 {
-	if ( m_visibilityUpdateAllowed )
-	{
-		m_isVisible = false;
-	}
-	timer->stop();
+    if ( m_visibilityUpdateAllowed )
+    {
+	m_isVisible = isVisible();
+    }
+    // !! could possibly stop polling now (idea: use someting like ref() and unref() on Mixer instance
+    //    Update: This is a stupid idea, because now the views are responsible for updating. So it will be done in the Views.
+    //    But the dock icon is currently no View, so that must be done first.
 }
 
 
@@ -668,7 +527,7 @@ KMixWindow::stopVisibilityUpdates() {
 
 void
 KMixWindow::slotHWInfo() {
-	KMessageBox::information( 0, m_hwInfoString, i18n("Mixer hardware information") );
+	KMessageBox::information( 0, m_hwInfoString, i18n("Mixer Hardware Information") );
 }
 
 void
