@@ -32,7 +32,6 @@
 #include <qmap.h>
 
 class QHBox;
-class QTimer;
 class QWidgetStack;
 
 // include files for KDE
@@ -66,7 +65,6 @@ KMixWindow : public KMainWindow
    void saveConfig();
    void loadConfig();
 
-   void initMixer();
    void initPrefDlg();
    void initActions();
    void initWidgets();
@@ -74,7 +72,7 @@ KMixWindow : public KMainWindow
 
    void updateDocking();
 
-	bool queryClose();
+   bool queryClose();
    void showEvent( QShowEvent * );
    void hideEvent( QHideEvent * );
 
@@ -84,13 +82,10 @@ KMixWindow : public KMainWindow
    void showHelp();
    void showAbout();
    void toggleMenuBar();
-   void loadVolumes();
+   //void loadVolumes();
    void saveVolumes();
    virtual void applyPrefs( KMixPrefDlg *prefDlg );
-   void updateDockIcon();
-   void updateDockTip(Volume vol);
    void stopVisibilityUpdates();
-   void slotConfigureKeys();
 
   private:
    KAccel *m_keyAccel;
@@ -103,37 +98,32 @@ KMixWindow : public KMainWindow
    bool m_hideOnClose;
    bool m_showTicks;
    bool m_showLabels;
+   bool m_onLogin;
    bool m_startVisible;
    bool m_showMenubar;
    bool m_isVisible;
-   bool m_multiDriverMode;
    bool m_visibilityUpdateAllowed;
-   int m_maxId;
+   bool m_multiDriverMode;         // Not officially supported. 
+   bool m_surroundView;            // Experimental. Off by defualt
+   Qt::Orientation m_toplevelOrientation;
 
    QPtrList<Mixer> m_mixers;
    QPtrList<KMixerWidget> m_mixerWidgets;
 
-   QTimer *m_layoutTimer;
-   int m_lockedLayout;
+   int m_lockedLayout; // !!
 
    QHBox* mixerNameLayout;
    KComboBox *m_cMixer;
    QWidgetStack *m_wsMixers;
    KMixPrefDlg *m_prefDlg;
    KMixDockWidget *m_dockWidget;
-   QTimer *timer;	// Timer for reading volume from HW
    QString m_hwInfoString;
-	QVBoxLayout *widgetsLayout;
-   //bool isCategoryUsed(Mixer* mixer, MixDevice::DeviceCategory categoryMask);
+   QVBoxLayout *widgetsLayout;
 
   private slots:
    //void removeMixerWidget( KMixerWidget *mw );
-   //void updateLayout();
-   void dockMute();
    void slotHWInfo();
-	void showSelectedMixer( int mixer );
-	void triggerUpdateLayout();
-	void updateLayoutNow();
+   void showSelectedMixer( int mixer );
 };
 
 #endif // KMIX_H
