@@ -22,23 +22,23 @@
 #ifndef KMIXAPPLET_H
 #define KMIXAPPLET_H
 
+// Qt
+#include <qlayout.h>
+#include <qptrlist.h>
 #include <qwidget.h>
-#include <kaction.h>
+
+// KDE
+#include <kaboutdata.h>
 #include <kdialogbase.h>
 #include <kpanelapplet.h>
-#include <qptrlist.h>
-#include <kcolordialog.h>
-#include <kaboutdata.h>
 
-#include "kmixerwidget.h"
+//KMix
+#include "viewapplet.h"
 
-class QTimer;
 class Mixer;
 class ColorWidget;
 class KMixApplet;
 
-// External main pointer
-extern KMixApplet *kmixApp;
 
 class AppletConfigDialog : public KDialogBase
 {
@@ -56,9 +56,6 @@ class AppletConfigDialog : public KDialogBase
    void setUseCustomColors(bool);
    bool useCustomColors() const;
 
-   void setReverseDirection(bool);
-   bool reverseDirection() const;
-
   protected slots:
    virtual void slotOk();
    virtual void slotApply();
@@ -74,52 +71,60 @@ class KMixApplet : public KPanelApplet
 {
    Q_OBJECT
 
-  public:
+public:
    KMixApplet( const QString& configFile, Type t = Normal,
 	       QWidget *parent = 0, const char *name = 0 );
    virtual ~KMixApplet();
 
-   int widthForHeight(int height) const;
-   int heightForWidth(int width) const; 
-   
+   struct Colors {
+       QColor high, low, back, mutedHigh, mutedLow, mutedBack;
+   };
+
    void about();
    void help();
    void preferences();   
    void reportBug();
+   void paletteChange ( const QPalette & oldPalette );
 
-  protected slots:
-   void triggerUpdateLayout();
-   void updateLayoutNow(); 
+   QSize sizeHint() const;
+   QSizePolicy sizePolicy() const;
+   int widthForHeight(int) const;
+   int heightForWidth(int) const;
+
+protected slots:
    void selectMixer();
    void applyPreferences();
    void preferencesDone();
+   void updateGeometrySlot();
   
-  protected:
+protected:
    void resizeEvent( QResizeEvent * );
    void saveConfig();
-    
-  private:
-   KMixerWidget *m_mixerWidget;
-   QPushButton *m_errorLabel;
-   QTimer *m_layoutTimer;
-   int m_lockedLayout;
-   AppletConfigDialog *m_pref;
-   bool reversedDir; //  reverses direction of sliders and icon position
+   void saveConfig( KConfig *config, const QString &grp );
+   void loadConfig();
+   void loadConfig( KConfig *config, const QString &grp );
+
+private:
    void positionChange(Position);
-   Direction checkReverse(Direction);
-   Direction getDirectionFromPositionHack(Position pos);
    void setColors();
+   void setColors( const Colors &color );
 
-   static int s_instCount;
-   static QPtrList<Mixer> *s_mixers;
-   static QTimer *s_timer;    
+   ViewApplet *m_mixerWidget;
+   QPushButton *m_errorLabel;
+   AppletConfigDialog *m_pref;
+
+   static int             s_instCount;
+   static QPtrList<Mixer> s_mixers;
+   Mixer                  *_mixer;
+
+   KMixApplet::Colors _colors;
+   bool               _customColors;
+
+   QLayout* _layout;
+
+   int     _mixerNum;
+   QString _mixerName;
 
-   KMixerWidget::Colors m_colors;
-   bool m_customColors;
-   
-   int mixerNum;
-   QString mixerName;
-   Mixer *mixer;
    KAboutData m_aboutData;
 };
 
