@@ -14,29 +14,28 @@ class Mixer_ALSA : public Mixer
 		~Mixer_ALSA();
 		
 		virtual int  readVolumeFromHW( int devnum, Volume &vol );
-		virtual int  writeVolumeToHW( int devnum, Volume vol );
+		virtual int  writeVolumeToHW( int devnum, Volume &vol );
 		virtual bool setRecsrcHW( int devnum, bool on);
 		virtual bool isRecsrcHW( int devnum );
+	        virtual void setEnumIdHW(int mixerIdx, unsigned int);
+      		virtual unsigned int enumIdHW(int mixerIdx);
+		virtual bool prepareUpdate();
+
 		
 	protected:
 		virtual int	openMixer();
 		virtual int releaseMixer();
 		
 	private:
-		snd_mixer_t *handle;
-#ifdef HAVE_ALSA_ASOUNDLIB_H
 		int identify( snd_mixer_selem_id_t *sid );
-		QString mixer_card_name;
-		QString mixer_device_name;
-		typedef QValueList<snd_mixer_elem_t *> AlsaMixerElemList;
-		AlsaMixerElemList mixer_elem_list;
-		
-#elif defined(HAVE_SYS_ASOUNDLIB_H)
-		snd_mixer_groups_t  groups;
-		snd_mixer_gid_t    *gid;
-		int numChannels( int mask );
-		int identify( int, const char* id );
-#endif
+		snd_mixer_elem_t* getMixerElem(int devnum);
+
+		virtual QString errorText(int mixer_error);
+		typedef QValueList<snd_mixer_selem_id_t *>AlsaMixerSidList;
+		AlsaMixerSidList mixer_sid_list;
+
+                bool _initialUpdate;
+		snd_mixer_t *_handle;
 };
 
 #endif
