@@ -1,9 +1,11 @@
 /*
    Kscd - A simple cd player for the KDE Project
 
-   $Id: kscd.h,v 1.103 2004/08/03 22:53:39 kernalex Exp $
+   $Id: kscd.h 379350 2005-01-17 09:49:15Z shaheed $
 
    Copyright (c) 1997 Bernd Johannes Wuebben math.cornell.edu
+   Copyright (c) 2002 Aaron J. Seigo <aseigo@kde.org>
+   Copyright (c) 2004 Alexander Kern <alex.kern@gmx.de>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,12 +20,8 @@
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
-
  */
 
-
-
 #ifndef __KSCD__
 #define __KSCD__
 
@@ -96,16 +94,6 @@ class KVolumeControl;
 
 using namespace KCDDB;
 
-struct mgconfigstruct
-{
-  int width;
-  int height;
-  int brightness;
-  int fadeMode;
-  bool pointsAreDiamonds;
-  double starSize;
-};
-
 typedef QValueList<int> RandomList;
 
 class KSCD : public kscdPanelDlg, public KSessionManaged, virtual public DCOPObject {
@@ -135,8 +123,12 @@ k_dcop:
     void volumeDown() { decVolume(); }
     void volumeUp() { incVolume(); }
     void setVolume(int v);
+    void setDevice(const QString& dev);
     int  getVolume() { return Prefs::volume(); }
     int currentTrack();
+    int currentTrackLength();
+    int currentPosition();
+    int getStatus();
     QString currentTrackTitle();
     QString currentAlbum();
     QString currentArtist();
@@ -156,7 +148,6 @@ public:
 
 signals:
     void trackChanged(const QString&);
-    void newServerList(const QStringList&);
 
 public slots:
     void setColors();
@@ -214,7 +205,7 @@ protected:
 
     void dragTime(int sec);
 
-    void closeEvent( QCloseEvent *e );
+    void closeEvent(QCloseEvent *e);
     void keyPressEvent( QKeyEvent* e );
     bool event( QEvent *e );
     //    void focusOutEvent(QFocusEvent *e);
@@ -226,11 +217,8 @@ protected:
 
     void updateDisplayedTrack(int track);
 
-    bool getArtist(QString& artist);
-
-    void clearSongList();
     void setSongListTo(int whichTrack);
-    void populateSongList();
+    void populateSongList(QString infoStatus);
     void updatePlayPB(bool playing);
 
     void updateConfigDialog(configWidget* widget);
@@ -242,7 +230,7 @@ private:
     QPopupMenu      *infoPopup;
 
     BW_LED_Number       *trackTimeLED[6];
-    
+
     QTimer              timer;
     QTimer              titlelabeltimer;
     QTimer              queryledtimer;
@@ -272,43 +260,30 @@ private:
     int                 prev_randomtrack();
     int                 real_randomtrack();
 
-  /**
-   * set the artist and title labels as well as the dock tooltip.
-   */
-    void setArtist(const QString& artist);
     void setTitle(int track);
 
-    QStringList     pathlist;
+    /**
+     * Info from CDDB, and exploded versions thereof.
+     */
+    KCDDB::CDInfo cddbInfo;
+    KCDDB::TrackOffsetList trackStartFrames;
     QStringList     tracktitlelist;
     QStringList     playlist;
-    QStringList     extlist;
-    QString         category;
-    QString         genre;
-    QFont           smallfont;
-    QFont           verysmallfont;
 
 // cddb support
 public slots:
-    void cddb_done(CDDB::Result);
-    void cddb_failed();
-    void cddb_no_info();
+    void cddb_failed(CDDB::Result);
     void mycddb_inexact_read();
     void CDDialogSelected();
     void CDDialogDone();
-    void get_cddb_info(bool);
+    void lookupCDDB();
+    void lookupCDDBDone(CDDB::Result);
 
 private:
-    void get_cdtext_info();
+    void lookupDevice();
 
 private:
     KCDDB::Client*  cddb;
-    QString         xmcd_data;
-    QStringList     cddbserverlist;
-    QString         current_server;
-    QStringList     cddbsubmitlist;
-    //bool            updateDialog;
-    int             revision;
-    int             year;
     KActionCollection* m_actions;
     KToggleAction* m_togglePopupsAction;
     KVolumeControl* m_volume;
