@@ -126,7 +126,7 @@ namespace KCDDB
       if (2 != tokenList.count())
       {
         if (!key.startsWith("EXT"))
-        continue;
+          continue;
       }
       else
         value = unescape ( tokenList[1].stripWhiteSpace() );
@@ -145,7 +145,7 @@ namespace KCDDB
       }
       else if ( "DGENRE" == key )
       {
-        genre = value;
+        genre += value;
       }
       else if ( "TTITLE" == key.left( 6 ) )
       {
@@ -210,15 +210,14 @@ namespace KCDDB
     }
 
     s += "DISCID=" + escape( id ) + "\n";
-    s += "DTITLE=" + escape( artist ) + " / " + escape( title ) + "\n";
+    s += createLine("DTITLE",escape( artist ) + " / " + escape( title ));
     s += "DYEAR=" + (0 == year ? QString::null : QString::number(year)) + "\n";
-    s += "DGENRE=" + escape( genre ) + "\n";
+    s += createLine("DGENRE",escape( genre ));
 
     for (uint i = 0; i < trackInfoList.count(); ++i)
     {
-      s += QString( "TTITLE%1=%2\n" )
-                .arg( i )
-                .arg( escape( trackInfoList[ i ].title ) );
+      s += createLine(QString("TTITLE%1").arg(i), 
+                escape( trackInfoList[ i ].title));
     }
 
     QStringList extdLines = QStringList::split('\n', extd, true);
@@ -226,7 +225,7 @@ namespace KCDDB
       s += QString("EXTD=\n");
     else
       for (QStringList::iterator it = extdLines.begin(); it != extdLines.end(); ++it)
-        s += "EXTD=" + escape( *it ) + "\n";
+        s += createLine("EXTD",escape( *it ));
 
     for (uint i = 0; i < trackInfoList.count(); ++i)
     {
@@ -235,9 +234,8 @@ namespace KCDDB
         s += QString( "EXTT%1=\n" ).arg( i );
       else
         for (QStringList::iterator it = lines.begin(); it != lines.end(); ++it)
-      s += QString( "EXTT%1=%2\n" )
-                .arg( i )
-                  .arg( escape( *it ) );
+          s += createLine(QString("EXTT%1").arg( i ),
+                  escape( *it ) );
     }
 
     s +="PLAYORDER=\n";
@@ -245,6 +243,30 @@ namespace KCDDB
     return s;
   }
 
+  // Creates a line in the form NAME=VALUE, and splits it into several
+  // lines if the line gets longer than 256 chars
+    QString
+  CDInfo::createLine(const QString& name, const QString& value) const
+  {
+    Q_ASSERT(name.length() < 254);
+
+    uint maxLength = 256 - name.length() - 2;
+
+    QString tmpValue = value;
+
+    QString lines;
+
+    while (tmpValue.length() > maxLength)
+      {
+      lines += QString("%1=%2\n").arg(name,tmpValue.left(maxLength));
+      tmpValue = tmpValue.mid(maxLength);
+    }
+
+    lines += QString("%1=%2\n").arg(name,tmpValue);
+
+    return lines;
+  }
+
     void
   CDInfo::checkTrack( uint trackNumber )
   {
