@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2000-2001 Charles Samuels <charles@kde.org>
+ * Copyright (c) 2000-2004 Charles Samuels <charles@kde.org>
  *               2000-2001 Neil Stevens <neil@qualityassistant.com>
  *
  * Copyright (c) from the patches of:
@@ -248,12 +248,14 @@ void SafeListViewItem::downloaded(int percent)
 
 void SafeListViewItem::downloadTimeout()
 {
-	setText(1, "-");
+	if (!removed)
+		setText(1, "-");
 }
 
 void SafeListViewItem::downloadFinished()
 {
-	setText(1, "");
+	if (!removed)
+		setText(1, "");
 }
 
 void SafeListViewItem::modified()
@@ -314,14 +316,16 @@ void SafeListViewItem::remove()
 	{
 		napp->player()->stop();
 		SPL->setCurrent(0);
-		napp->player()->playCurrent();
 	}
 	else if (napp->player()->current()==this)
 	{
 //		SPL->setCurrent(0);
 //		napp->player()->playCurrent();
-		if (!SPL->exiting())
+
+		if (napp->player()->isPlaying() && !SPL->exiting())
 			napp->player()->forward();
+		else
+			SPL->setCurrent(0);
 	}
 
 	if (listView())
@@ -336,6 +340,7 @@ void SafeListViewItem::remove()
 		SPL->setCurrent(0);
 	}
 
+	dequeue();
 	PlaylistItemData::removed();
 }
 
@@ -418,20 +423,24 @@ void List::keyPressEvent(QKeyEvent *e)
 class NoatunSaver : public PlaylistSaver
 {
 	List *mList;
-	SafeListViewItem *after;
+	SafeListViewItem *after, *mFirst;
 public:
 	NoatunSaver(List *l, QListViewItem *after=0)
 		: mList(l)
 	{
-		this->after= static_cast<SafeListViewItem*>(after);
+		this->after = static_cast<SafeListViewItem*>(after);
+		mFirst = 0;
 	}
 
-	QListViewItem *getAfter() { return after;}
+	QListViewItem *getAfter() { return after; }
+	QListViewItem *getFirst() { return mFirst; }
 
 protected:
 	virtual void readItem(const QMap<QString,QString> &properties)
 	{
-		after=new SafeListViewItem(mList, after, properties);
+		after = new SafeListViewItem(mList, after, properties);
+		if (mFirst==0)
+			mFirst = after;
 	}
 
 	virtual PlaylistItem writeItem()
@@ -506,6 +515,12 @@ QListViewItem *List::importGlobal(const KURL &u, QListViewItem *after)
 		return after;
 	}
 
+	// return the first item added from this playlist
+	// that way noatun can start playing the first item
+	if (saver.getFirst())
+		return saver.getFirst();
+
+	// failsafe in case nothing was added, getFirst() may return 0
 	return saver.getAfter();
 }
 
@@ -521,7 +536,10 @@ QListViewItem *List::addFile(const KURL& url, bool play, QListViewItem *after)
 		)
 	{
 		// a playlist is requested
-		return importGlobal(url, after);
+		QListViewItem *i = importGlobal(url, after);
+		if (play)
+			SPL->listItemSelected(i);
+		return i;
 	}
 	else
 	{
@@ -534,7 +552,7 @@ QListViewItem *List::addFile(const KURL& url, bool play, QListViewItem *after)
 		}
 		else
 		{
-			QListViewItem *i=new SafeListViewItem(this, after, url);
+			QListViewItem *i = new SafeListViewItem(this, after, url);
 			if (play)
 				SPL->listItemSelected(i);
 			return i;
@@ -616,7 +634,7 @@ void List::slotRedirection(KIO::Job *, const KURL & url)
 /////////////////////////////////
 
 View::View(SplitPlaylist *)
-	: KMainWindow(0,0)
+	: KMainWindow(0, "NoatunSplitplaylistView")
 {
 	list=new List(this);
 	setCentralWidget(list);
@@ -638,7 +656,7 @@ View::View(SplitPlaylist *)
 	mSaveAs=KStdAction::saveAs(this, SLOT(saveAs()), actionCollection());
 
 	(void) new KAction(i18n("Shuffle"), "misc", 0, SPL, SLOT( randomize() ), actionCollection(), "shuffle");
-	(void) new KAction(i18n("Clear"), "fileclose", 0, list, SLOT( clear() ), actionCollection(), "clear");
+	(void) new KAction(i18n("Clear"), "editclear", 0, list, SLOT( clear() ), actionCollection(), "clear");
 
 	createGUI("splui.rc");
 
@@ -824,24 +842,18 @@ void View::saveAs()
 
 void View::open()
 {
-	KURL u=KFileDialog::getOpenURL(0, "*.xml splitplaylistdata *.pls *.m3u\n*", this, i18n("Insert Playlist"));
+	KURL u=KFileDialog::getOpenURL(0, "*.xml splitplaylistdata *.pls *.m3u\n*", this, i18n("Open Playlist"));
 	if(!u.isValid())
 		return;
+	mPlaylistFile = u;
 	list->openGlobal(u);
 	setModified(false);
 }
 
 void View::openNew()
 {
-	KURL u=KFileDialog::getOpenURL(0, "*.xml splitplaylistdata *.pls *.m3u\n*", this, i18n("Open New Playlist"));
-	if(!u.isValid())
-		return;
-	mPlaylistFile = u;
-
+	mPlaylistFile = "";
 	listView()->clear();
-	list->openGlobal(mPlaylistFile);
-
-	setModified(false);
 }
 
 void List::clear()
@@ -854,8 +866,27 @@ void List::clear()
 void View::deleteSelected()
 {
 	QPtrList<QListViewItem> items(list->selectedItems());
+
+	bool stopped=false;
+	// noatun shouldn't play files for now
+	QListViewItem *afterLast=0;
+
 	for (QPtrListIterator<QListViewItem> it(items); it.current(); ++it)
-		static_cast<SafeListViewItem*>(*it)->remove();
+	{
+		SafeListViewItem *i = static_cast<SafeListViewItem*>(*it);
+		if (!stopped && SPL->current() == i)
+		{
+			napp->player()->stop();
+			SPL->setCurrent(0);
+			stopped = true;
+		}
+		i->remove();
+
+		afterLast = i->itemBelow();
+	}
+
+	if (stopped)
+		SPL->setCurrent(static_cast<SafeListViewItem*>(afterLast));
 
 	setModified(true);
 }
